# coding:utf-8

from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver

from kinder import settings as kinder_settings
from kinder.core.dict.models import (
    GroupAgeSubCathegory, HealthNeed, UnitKind, DouType)
from kinder.core.privilege.models import Privilege
from kinder.core.unit.models import Unit, UnitStatus
from kinder.core.declaration.models import Declaration
from kinder.core.declaration_status.signals import declaration_status_change
from concentrator.dict.proxy import (
    GroupAgeSubCathegoryProxy, HealthNeedProxy, PrivilegeProxy, UnitProxy)
from concentrator.dict.signal_handlers import (
    obj_save_handler, obj_delete_handler)
from concentrator.tasks import (
    SendUpdateApplicationState)

#-------------------------------------------------------------------------------
# Для модели Организации более сложное поведение,
# создание отправляем:
# - при создании c типом ДОО
# - при смене типа на ДОО
# обновление отправляем:
# - если были внесены изменения в организацию с типом ДОО
# удаление отправляем:
# - при удалении организации с типом ДОО
# - при смене типа с ДОО на любой другой
#-------------------------------------------------------------------------------
@receiver(post_save, sender=Unit)
def sign_unit_save(sender, instance, **kwargs):
    """
    В концентратор уходят только организации с типом ДОО
    , при создании с типом ДОО, при смене типа с любого другого на ДОО
    , с разрешением на отображение на портале, пустым либо гос. типом ДОО
    @param sender:
    @param instance:
    @param kwargs:
    @return:
    """

    unit_statuses_for_delete = (
        UnitStatus.IN_CAPITAL_REPAIR,
        UnitStatus.RECONSTRUCTION,
        UnitStatus.SUSPENDED,
        UnitStatus.PENDING_OPEN,
        UnitStatus.LIQUIDATED,
        UnitStatus.CLOSED,
        UnitStatus.JOINED_OTHER
    )

    def is_for_ignore(instance):
        """
        Проверка следует ли игнорировать организацию
        Условия:
            Запрет на отображение на портале
            Тип ДОО не гос. (только при включенном SMEV_ONLY_GOV_DOU_OR_EMPTY)
        """

        result = (
            # Если запрет показа на портале
            instance.is_not_show_on_poral or
            # Или статус организации который нельзя отправлять на портал
            instance.status in unit_statuses_for_delete
        )

        if kinder_settings.SMEV_ONLY_GOV_DOU_OR_EMPTY:
            # Если тип ДОО не None или не гос.
            result |= (instance.dou_type_id is not None
                       and instance.dou_type.code not in
                       DouType.GOVERNMENT_TYPES)

        return result

    def is_for_delete(instance):
        """
        Проверка следует ли посылать запрос на удаление организации
        Условия:
            Запрет на отображение на портале
            Тип ДОО не гос. (только при включенном SMEV_ONLY_GOV_DOU_OR_EMPTY)
        """
        previous_status = previous_state['status']
        previous_is_not_show_on_poral = previous_state['is_not_show_on_poral']
        original_dou_type_id = previous_state['dou_type_id']

        # Если была смена запрета на показ на портале False -> True
        result = (instance.is_not_show_on_poral
                  and not previous_is_not_show_on_poral)

        result = result or (
            # Если статус был который можно было отправлять на портал
            previous_status not in unit_statuses_for_delete and
            # И установили тот что нельзя
            instance.status in unit_statuses_for_delete
        )

        if kinder_settings.SMEV_ONLY_GOV_DOU_OR_EMPTY:
            # Если была смена типа ДОО (None, гос.) -> любой другой
            result |= ((instance.dou_type_id is not None
                       and instance.dou_type.code
                        not in DouType.GOVERNMENT_TYPES)
                       and instance.dou_type_id != original_dou_type_id)

            # Если ранее не было случаев подпадающих под удаление
            result &= not (
                (instance.is_not_show_on_poral ==
                 previous_is_not_show_on_poral == True)
                or ((instance.dou_type_id is not None
                and instance.dou_type.code not in DouType.GOVERNMENT_TYPES)
                and instance.dou_type_id == original_dou_type_id))

        return result

    def is_for_create(instance):
        """
        Проверка следует ли посылать запрос на создание организации
        """
        previous_status = previous_state['status']
        previous_is_not_show_on_poral = previous_state['is_not_show_on_poral']
        original_dou_type_id = previous_state['dou_type_id']

        # Если была смена запрета на показ на портале True -> False
        result = (not instance.is_not_show_on_poral
                  and previous_is_not_show_on_poral)

        result = result or (
            # Если раньше статус был который нельзя показывать на портале
            previous_status in unit_statuses_for_delete and
            # И заменили на тот что можно
            instance.status not in unit_statuses_for_delete
        )

        if kinder_settings.SMEV_ONLY_GOV_DOU_OR_EMPTY:
            # Если была смена типа ДОО любой другой -> (None, гос.)
            result |= ((instance.dou_type_id is None
                       or instance.dou_type.code in DouType.GOVERNMENT_TYPES)
                       and
                       instance.dou_type_id != original_dou_type_id)

        return result

    if not kwargs['created']:
        previous_state = instance.previous_state()
        original_kind = previous_state['kind_id']
        # Создание ДОО при смене типа организации на ДОО
        if (instance.kind_id == UnitKind.DOU
            and original_kind != UnitKind.DOU):

            if not is_for_ignore(instance):
                obj_save_handler(instance, UnitProxy, True)
        # Обновление ДОО при внесении изменений в организацию с типом ДОО
        elif (instance.kind_id == UnitKind.DOU
            and original_kind == UnitKind.DOU):

            if is_for_delete(instance):
                obj_delete_handler(instance, UnitProxy)
            elif not is_for_ignore(instance):
                obj_save_handler(instance, UnitProxy, is_for_create(instance))
        # Удаление ДОО при смене типа с ДОО на любой другой
        elif (instance.kind_id != UnitKind.DOU
              and original_kind == UnitKind.DOU):
            obj_delete_handler(instance, UnitProxy)

    # Создание ДОО, если тип организации ДОО
    elif (kwargs['created']
          and instance.kind_id == UnitKind.DOU
          and not is_for_ignore(instance)):
        obj_save_handler(instance, UnitProxy, True)


@receiver(post_delete, sender=Unit)
def sign_unit_delete(sender, instance, **kwargs):
    # удаление организации с типов ДОО
    if instance.kind_id == UnitKind.DOU:
        obj_delete_handler(instance, UnitProxy)


#-------------------------------------------------------------------------------
# Сигналы на создания/изменения в справочниках
#-------------------------------------------------------------------------------
@receiver(post_save, sender=GroupAgeSubCathegory)
def sign_group_age_sub_cathegory_save(sender, instance, **kwargs):
    obj_save_handler(instance, GroupAgeSubCathegoryProxy, kwargs['created'])


@receiver(post_save, sender=HealthNeed)
def sign_health_need_save(sender, instance, **kwargs):
    obj_save_handler(instance, HealthNeedProxy, kwargs['created'])


@receiver(post_save, sender=Privilege)
def sign_privilege_save(sender, instance, **kwargs):
    obj_save_handler(instance, PrivilegeProxy, kwargs['created'])

#-------------------------------------------------------------------------------
# Сигналы на удаления в справочниках
#-------------------------------------------------------------------------------
@receiver(post_delete, sender=GroupAgeSubCathegory)
def sign_group_age_sub_cathegory_delete(sender, instance, **kwargs):
    obj_delete_handler(instance, GroupAgeSubCathegoryProxy)


@receiver(post_delete, sender=HealthNeed)
def sign_health_need_delete(sender, instance, **kwargs):
    obj_delete_handler(instance, HealthNeedProxy)


@receiver(post_delete, sender=Privilege)
def sign_privilege_delete(sender, instance, **kwargs):
    obj_delete_handler(instance, PrivilegeProxy)


@receiver(declaration_status_change, sender=Declaration)
def update_application_state(**kwargs):
    declaration = kwargs['declaration']
    is_auto = kwargs.get('is_auto')
    commentary = kwargs.get('commentary')
    if declaration.client_id:
        SendUpdateApplicationState().apply_async(
            (),
            {'declaration_id': declaration.id, 'is_auto': is_auto,
             'commentary': commentary})
